@@ -16,10 +16,11 @@
 Provider related utilities
 """
 
-from libcloud.utils.misc import get_driver as _get_provider_driver
-from libcloud.utils.misc import set_driver as _set_provider_driver
-from libcloud.compute.types import Provider, DEPRECATED_RACKSPACE_PROVIDERS
+from libcloud.compute.types import Provider
+from libcloud.common.providers import get_driver as _get_provider_driver
+from libcloud.common.providers import set_driver as _set_provider_driver
 from libcloud.compute.types import OLD_CONSTANT_TO_NEW_MAPPING
+from libcloud.compute.deprecated import DEPRECATED_DRIVERS
 
 __all__ = [
     "Provider",
@@ -29,44 +30,16 @@ __all__ = [
 DRIVERS = {
     Provider.AZURE:
     ('libcloud.compute.drivers.azure', 'AzureNodeDriver'),
+    Provider.AZURE_ARM:
+    ('libcloud.compute.drivers.azure_arm', 'AzureNodeDriver'),
     Provider.DUMMY:
     ('libcloud.compute.drivers.dummy', 'DummyNodeDriver'),
-    Provider.EC2_US_EAST:
+    Provider.EC2:
     ('libcloud.compute.drivers.ec2', 'EC2NodeDriver'),
-    Provider.EC2_EU_WEST:
-    ('libcloud.compute.drivers.ec2', 'EC2EUNodeDriver'),
-    Provider.EC2_US_WEST:
-    ('libcloud.compute.drivers.ec2', 'EC2USWestNodeDriver'),
-    Provider.EC2_US_WEST_OREGON:
-    ('libcloud.compute.drivers.ec2', 'EC2USWestOregonNodeDriver'),
-    Provider.EC2_AP_SOUTHEAST:
-    ('libcloud.compute.drivers.ec2', 'EC2APSENodeDriver'),
-    Provider.EC2_AP_NORTHEAST:
-    ('libcloud.compute.drivers.ec2', 'EC2APNENodeDriver'),
-    Provider.EC2_SA_EAST:
-    ('libcloud.compute.drivers.ec2', 'EC2SAEastNodeDriver'),
-    Provider.EC2_AP_SOUTHEAST2:
-    ('libcloud.compute.drivers.ec2', 'EC2APSESydneyNodeDriver'),
     Provider.ECP:
     ('libcloud.compute.drivers.ecp', 'ECPNodeDriver'),
     Provider.ELASTICHOSTS:
     ('libcloud.compute.drivers.elastichosts', 'ElasticHostsNodeDriver'),
-    Provider.ELASTICHOSTS_UK1:
-    ('libcloud.compute.drivers.elastichosts', 'ElasticHostsUK1NodeDriver'),
-    Provider.ELASTICHOSTS_UK2:
-    ('libcloud.compute.drivers.elastichosts', 'ElasticHostsUK2NodeDriver'),
-    Provider.ELASTICHOSTS_US1:
-    ('libcloud.compute.drivers.elastichosts', 'ElasticHostsUS1NodeDriver'),
-    Provider.ELASTICHOSTS_US2:
-    ('libcloud.compute.drivers.elastichosts', 'ElasticHostsUS2NodeDriver'),
-    Provider.ELASTICHOSTS_US3:
-    ('libcloud.compute.drivers.elastichosts', 'ElasticHostsUS3NodeDriver'),
-    Provider.ELASTICHOSTS_CA1:
-    ('libcloud.compute.drivers.elastichosts', 'ElasticHostsCA1NodeDriver'),
-    Provider.ELASTICHOSTS_AU1:
-    ('libcloud.compute.drivers.elastichosts', 'ElasticHostsAU1NodeDriver'),
-    Provider.ELASTICHOSTS_CN1:
-    ('libcloud.compute.drivers.elastichosts', 'ElasticHostsCN1NodeDriver'),
     Provider.SKALICLOUD:
     ('libcloud.compute.drivers.skalicloud', 'SkaliCloudNodeDriver'),
     Provider.SERVERLOVE:
@@ -81,8 +54,6 @@ DRIVERS = {
     ('libcloud.compute.drivers.rackspace', 'RackspaceNodeDriver'),
     Provider.RACKSPACE_FIRST_GEN:
     ('libcloud.compute.drivers.rackspace', 'RackspaceFirstGenNodeDriver'),
-    Provider.HPCLOUD:
-    ('libcloud.compute.drivers.hpcloud', 'HPCloudNodeDriver'),
     Provider.KILI:
     ('libcloud.compute.drivers.kili', 'KiliCloudNodeDriver'),
     Provider.VPSNET:
@@ -97,8 +68,6 @@ DRIVERS = {
     ('libcloud.compute.drivers.softlayer', 'SoftLayerNodeDriver'),
     Provider.EUCALYPTUS:
     ('libcloud.compute.drivers.ec2', 'EucNodeDriver'),
-    Provider.IBM:
-    ('libcloud.compute.drivers.ibm_sce', 'IBMNodeDriver'),
     Provider.OPENNEBULA:
     ('libcloud.compute.drivers.opennebula', 'OpenNebulaNodeDriver'),
     Provider.BRIGHTBOX:
@@ -109,14 +78,10 @@ DRIVERS = {
     ('libcloud.compute.drivers.bluebox', 'BlueboxNodeDriver'),
     Provider.GANDI:
     ('libcloud.compute.drivers.gandi', 'GandiNodeDriver'),
-    Provider.OPSOURCE:
-    ('libcloud.compute.drivers.opsource', 'OpsourceNodeDriver'),
     Provider.DIMENSIONDATA:
     ('libcloud.compute.drivers.dimensiondata', 'DimensionDataNodeDriver'),
     Provider.OPENSTACK:
     ('libcloud.compute.drivers.openstack', 'OpenStackNodeDriver'),
-    Provider.NINEFOLD:
-    ('libcloud.compute.drivers.ninefold', 'NinefoldNodeDriver'),
     Provider.VCLOUD:
     ('libcloud.compute.drivers.vcloud', 'VCloudNodeDriver'),
     Provider.TERREMARK:
@@ -139,8 +104,6 @@ DRIVERS = {
     ('libcloud.compute.drivers.digitalocean', 'DigitalOceanNodeDriver'),
     Provider.NEPHOSCALE:
     ('libcloud.compute.drivers.nephoscale', 'NephoscaleNodeDriver'),
-    Provider.CLOUDFRAMES:
-    ('libcloud.compute.drivers.cloudframes', 'CloudFramesNodeDriver'),
     Provider.EXOSCALE:
     ('libcloud.compute.drivers.exoscale', 'ExoscaleNodeDriver'),
     Provider.IKOULA:
@@ -163,31 +126,33 @@ DRIVERS = {
     ('libcloud.compute.drivers.packet', 'PacketNodeDriver'),
     Provider.ONAPP:
     ('libcloud.compute.drivers.onapp', 'OnAppNodeDriver'),
-    Provider.RUNABOVE:
-    ('libcloud.compute.drivers.runabove', 'RunAboveNodeDriver'),
-
-    # Deprecated
-    Provider.CLOUDSIGMA_US:
-    ('libcloud.compute.drivers.cloudsigma', 'CloudSigmaLvsNodeDriver'),
+    Provider.OVH:
+    ('libcloud.compute.drivers.ovh', 'OvhNodeDriver'),
+    Provider.INTERNETSOLUTIONS:
+    ('libcloud.compute.drivers.internetsolutions',
+     'InternetSolutionsNodeDriver'),
+    Provider.INDOSAT:
+    ('libcloud.compute.drivers.indosat', 'IndosatNodeDriver'),
+    Provider.MEDONE:
+    ('libcloud.compute.drivers.medone', 'MedOneNodeDriver'),
+    Provider.BSNL:
+    ('libcloud.compute.drivers.bsnl', 'BSNLNodeDriver'),
+    Provider.NTTA:
+    ('libcloud.compute.drivers.ntta', 'NTTAmericaNodeDriver'),
+    Provider.ALIYUN_ECS:
+    ('libcloud.compute.drivers.ecs', 'ECSDriver'),
+    Provider.CLOUDSCALE:
+    ('libcloud.compute.drivers.cloudscale', 'CloudscaleNodeDriver'),
 }
 
 
 def get_driver(provider):
-    if provider in DEPRECATED_RACKSPACE_PROVIDERS:
-        id_to_name_map = dict([(v, k) for k, v in Provider.__dict__.items()])
-        old_name = id_to_name_map[provider]
-        new_name = id_to_name_map[OLD_CONSTANT_TO_NEW_MAPPING[provider]]
-
-        url = 'http://s.apache.org/lc0140un'
-        msg = ('Provider constant %s has been removed. New constant '
-               'is now called %s.\n'
-               'For more information on this change and how to modify your '
-               'code to work with it, please visit: %s' %
-               (old_name, new_name, url))
-        raise Exception(msg)
-
-    return _get_provider_driver(DRIVERS, provider)
+    deprecated_constants = OLD_CONSTANT_TO_NEW_MAPPING
+    return _get_provider_driver(drivers=DRIVERS, provider=provider,
+                                deprecated_providers=DEPRECATED_DRIVERS,
+                                deprecated_constants=deprecated_constants)
 
 
 def set_driver(provider, module, klass):
-    return _set_provider_driver(DRIVERS, provider, module, klass)
+    return _set_provider_driver(drivers=DRIVERS, provider=provider,
+                                module=module, klass=klass)
