@@ -30,123 +30,165 @@ __all__ = [
     "MalformedResponseError",
     "InvalidCredsError",
     "InvalidCredsException",
-    "DEPRECATED_RACKSPACE_PROVIDERS",
+
     "OLD_CONSTANT_TO_NEW_MAPPING"
 ]
 
 
-class Provider(object):
+class Type(object):
+    @classmethod
+    def tostring(cls, value):
+        """Return the string representation of the state object attribute
+        :param str value: the state object to turn into string
+        :return: the uppercase string that represents the state object
+        :rtype: str
+        """
+        return value.upper()
+
+    @classmethod
+    def fromstring(cls, value):
+        """Return the state object attribute that matches the string
+        :param str value: the string to look up
+        :return: the state object attribute that matches the string
+        :rtype: str
+        """
+        return getattr(cls, value.upper(), None)
+
+
+class Provider(Type):
     """
     Defines for each of the supported providers
 
+    Non-Dummy drivers are sorted in alphabetical order. Please preserve this
+    ordering when adding new drivers.
+
     :cvar DUMMY: Example provider
-    :cvar EC2_US_EAST: Amazon AWS US N. Virgina
-    :cvar EC2_US_WEST: Amazon AWS US N. California
-    :cvar EC2_EU_WEST: Amazon AWS EU Ireland
-    :cvar RACKSPACE: Rackspace next-gen OpenStack based Cloud Servers
-    :cvar RACKSPACE_FIRST_GEN: Rackspace First Gen Cloud Servers
+    :cvar ABIQUO: Abiquo driver
+    :cvar ALIYUN_ECS: Aliyun ECS driver.
+    :cvar AURORACOMPUTE: Aurora Compute driver.
+    :cvar AZURE: Azure (classic) driver.
+    :cvar AZURE_ARM: Azure Resource Manager (modern) driver.
+    :cvar BLUEBOX: Bluebox
+    :cvar CLOUDSIGMA: CloudSigma
+    :cvar CLOUDSCALE: cloudscale.ch
+    :cvar CLOUDSTACK: CloudStack
+    :cvar DIMENSIONDATA: Dimension Data Cloud
+    :cvar EC2: Amazon AWS.
+    :cvar ECP: Enomaly
+    :cvar ELASTICHOSTS: ElasticHosts.com
+    :cvar EXOSCALE: Exoscale driver.
     :cvar GCE: Google Compute Engine
     :cvar GOGRID: GoGrid
-    :cvar VPSNET: VPS.net
-    :cvar LINODE: Linode.com
-    :cvar VCLOUD: vmware vCloud
-    :cvar RIMUHOSTING: RimuHosting.com
-    :cvar ECP: Enomaly
-    :cvar IBM: IBM Developer Cloud
-    :cvar OPENNEBULA: OpenNebula.org
-    :cvar ELASTICHOSTS: ElasticHosts.com
-    :cvar CLOUDSIGMA: CloudSigma
-    :cvar NIMBUS: Nimbus
-    :cvar BLUEBOX: Bluebox
-    :cvar OPSOURCE: Opsource Cloud
-    :cvar DIMENSIONDATA: Dimension Data Cloud
-    :cvar NINEFOLD: Ninefold
-    :cvar TERREMARK: Terremark
-    :cvar EC2_US_WEST_OREGON: Amazon AWS US West 2 (Oregon)
-    :cvar CLOUDSTACK: CloudStack
-    :cvar CLOUDSIGMA_US: CloudSigma US Las Vegas
-    :cvar LIBVIRT: Libvirt driver
-    :cvar JOYENT: Joyent driver
-    :cvar VCL: VCL driver
-    :cvar KTUCLOUD: kt ucloud driver
     :cvar GRIDSPOT: Gridspot driver
-    :cvar ABIQUO: Abiquo driver
-    :cvar NEPHOSCALE: NephoScale driver
-    :cvar EXOSCALE: Exoscale driver.
+    :cvar IBM: IBM Developer Cloud
     :cvar IKOULA: Ikoula driver.
-    :cvar OUTSCALE_SAS: Outscale SAS driver.
+    :cvar JOYENT: Joyent driver
+    :cvar KTUCLOUD: kt ucloud driver
+    :cvar LIBVIRT: Libvirt driver
+    :cvar LINODE: Linode.com
+    :cvar NEPHOSCALE: NephoScale driver
+    :cvar NIMBUS: Nimbus
+    :cvar NINEFOLD: Ninefold
+    :cvar OPENNEBULA: OpenNebula.org
+    :cvar OPSOURCE: Opsource Cloud
     :cvar OUTSCALE_INC: Outscale INC driver.
+    :cvar OUTSCALE_SAS: Outscale SAS driver.
     :cvar PROFIT_BRICKS: ProfitBricks driver.
+    :cvar RACKSPACE: Rackspace next-gen OpenStack based Cloud Servers
+    :cvar RACKSPACE_FIRST_GEN: Rackspace First Gen Cloud Servers
+    :cvar RIMUHOSTING: RimuHosting.com
+    :cvar TERREMARK: Terremark
+    :cvar VCL: VCL driver
+    :cvar VCLOUD: vmware vCloud
+    :cvar VPSNET: VPS.net
     :cvar VULTR: vultr driver.
-    :cvar AZURE: Azure driver.
-    :cvar AURORACOMPUTE: Aurora Compute driver.
     """
     AZURE = 'azure'
+    AZURE_ARM = 'azure_arm'
     DUMMY = 'dummy'
-    EC2 = 'ec2_us_east'
-    RACKSPACE = 'rackspace'
+    ABIQUO = 'abiquo'
+    ALIYUN_ECS = 'aliyun_ecs'
+    AURORACOMPUTE = 'aurora_compute'
+    AZURE = 'azure'
+    BLUEBOX = 'bluebox'
+    BRIGHTBOX = 'brightbox'
+    BSNL = 'bsnl'
+    CISCOCCS = 'ciscoccs'
+    CLOUDFRAMES = 'cloudframes'
+    CLOUDSIGMA = 'cloudsigma'
+    CLOUDSCALE = 'cloudscale'
+    CLOUDSTACK = 'cloudstack'
+    CLOUDWATT = 'cloudwatt'
+    DIGITAL_OCEAN = 'digitalocean'
+    DIMENSIONDATA = 'dimensiondata'
+    EC2 = 'ec2'
+    ECP = 'ecp'
+    ELASTICHOSTS = 'elastichosts'
+    EUCALYPTUS = 'eucalyptus'
+    EXOSCALE = 'exoscale'
+    GANDI = 'gandi'
     GCE = 'gce'
     GOGRID = 'gogrid'
-    VPSNET = 'vpsnet'
-    LINODE = 'linode'
-    VCLOUD = 'vcloud'
-    RIMUHOSTING = 'rimuhosting'
-    VOXEL = 'voxel'
-    SOFTLAYER = 'softlayer'
-    EUCALYPTUS = 'eucalyptus'
-    ECP = 'ecp'
-    IBM = 'ibm'
-    OPENNEBULA = 'opennebula'
-    ELASTICHOSTS = 'elastichosts'
-    BRIGHTBOX = 'brightbox'
-    CLOUDSIGMA = 'cloudsigma'
-    NIMBUS = 'nimbus'
-    BLUEBOX = 'bluebox'
-    GANDI = 'gandi'
-    OPSOURCE = 'opsource'
-    DIMENSIONDATA = 'dimensiondata'
-    OPENSTACK = 'openstack'
-    SKALICLOUD = 'skalicloud'
-    SERVERLOVE = 'serverlove'
-    NINEFOLD = 'ninefold'
-    TERREMARK = 'terremark'
-    CLOUDSTACK = 'cloudstack'
-    LIBVIRT = 'libvirt'
-    JOYENT = 'joyent'
-    VCL = 'vcl'
-    KTUCLOUD = 'ktucloud'
     GRIDSPOT = 'gridspot'
-    RACKSPACE_FIRST_GEN = 'rackspace_first_gen'
     HOSTVIRTUAL = 'hostvirtual'
-    ABIQUO = 'abiquo'
-    DIGITAL_OCEAN = 'digitalocean'
-    NEPHOSCALE = 'nephoscale'
-    CLOUDFRAMES = 'cloudframes'
-    EXOSCALE = 'exoscale'
+    IBM = 'ibm'
     IKOULA = 'ikoula'
-    OUTSCALE_SAS = 'outscale_sas'
+    INDOSAT = 'indosat'
+    INTERNETSOLUTIONS = 'internetsolutions'
+    JOYENT = 'joyent'
+    KTUCLOUD = 'ktucloud'
+    LIBVIRT = 'libvirt'
+    LINODE = 'linode'
+    MEDONE = 'medone'
+    NEPHOSCALE = 'nephoscale'
+    NIMBUS = 'nimbus'
+    NINEFOLD = 'ninefold'
+    NTTA = 'ntta'
+    OPENNEBULA = 'opennebula'
+    OPENSTACK = 'openstack'
+    OPSOURCE = 'opsource'
     OUTSCALE_INC = 'outscale_inc'
-    VSPHERE = 'vsphere'
-    PROFIT_BRICKS = 'profitbricks'
-    VULTR = 'vultr'
-    AURORACOMPUTE = 'aurora_compute'
-    CLOUDWATT = 'cloudwatt'
+    OUTSCALE_SAS = 'outscale_sas'
+    OVH = 'ovh'
     PACKET = 'packet'
+    PROFIT_BRICKS = 'profitbricks'
+    RACKSPACE = 'rackspace'
+    RACKSPACE_FIRST_GEN = 'rackspace_first_gen'
+    RIMUHOSTING = 'rimuhosting'
     RUNABOVE = 'runabove'
+    SERVERLOVE = 'serverlove'
+    SKALICLOUD = 'skalicloud'
+    SOFTLAYER = 'softlayer'
+    TERREMARK = 'terremark'
+    VCL = 'vcl'
+    VCLOUD = 'vcloud'
+    VOXEL = 'voxel'
+    VPSNET = 'vpsnet'
+    VSPHERE = 'vsphere'
+    VULTR = 'vultr'
 
     # OpenStack based providers
-    HPCLOUD = 'hpcloud'
     CLOUDWATT = 'cloudwatt'
+    HPCLOUD = 'hpcloud'
     KILI = 'kili'
     ONAPP = 'onapp'
 
-    # Deprecated constants which are still supported
+    # Deprecated constants which aren't supported anymore
+    RACKSPACE_UK = 'rackspace_uk'
+    RACKSPACE_NOVA_BETA = 'rackspace_nova_beta'
+    RACKSPACE_NOVA_DFW = 'rackspace_nova_dfw'
+    RACKSPACE_NOVA_LON = 'rackspace_nova_lon'
+    RACKSPACE_NOVA_ORD = 'rackspace_nova_ord'
+
     EC2_US_EAST = 'ec2_us_east'
+    EC2_US_EAST_OHIO = 'ec2_us_east_ohio'
     EC2_EU = 'ec2_eu_west'  # deprecated name
     EC2_EU_WEST = 'ec2_eu_west'
     EC2_US_WEST = 'ec2_us_west'
     EC2_AP_SOUTHEAST = 'ec2_ap_southeast'
     EC2_AP_NORTHEAST = 'ec2_ap_northeast'
+    EC2_AP_NORTHEAST1 = 'ec2_ap_northeast_1'
+    EC2_AP_NORTHEAST2 = 'ec2_ap_northeast_2'
     EC2_US_WEST_OREGON = 'ec2_us_west_oregon'
     EC2_SA_EAST = 'ec2_sa_east'
     EC2_AP_SOUTHEAST2 = 'ec2_ap_southeast_2'
@@ -162,13 +204,6 @@ class Provider(object):
 
     CLOUDSIGMA_US = 'cloudsigma_us'
 
-    # Deprecated constants which aren't supported anymore
-    RACKSPACE_UK = 'rackspace_uk'
-    RACKSPACE_NOVA_BETA = 'rackspace_nova_beta'
-    RACKSPACE_NOVA_DFW = 'rackspace_nova_dfw'
-    RACKSPACE_NOVA_LON = 'rackspace_nova_lon'
-    RACKSPACE_NOVA_ORD = 'rackspace_nova_ord'
-
     # Removed
     # SLICEHOST = 'slicehost'
 
@@ -179,81 +214,101 @@ DEPRECATED_RACKSPACE_PROVIDERS = [Provider.RACKSPACE_UK,
                                   Provider.RACKSPACE_NOVA_LON,
                                   Provider.RACKSPACE_NOVA_ORD]
 OLD_CONSTANT_TO_NEW_MAPPING = {
-    Provider.RACKSPACE: Provider.RACKSPACE_FIRST_GEN,
+    # Rackspace
     Provider.RACKSPACE_UK: Provider.RACKSPACE_FIRST_GEN,
 
     Provider.RACKSPACE_NOVA_BETA: Provider.RACKSPACE,
     Provider.RACKSPACE_NOVA_DFW: Provider.RACKSPACE,
     Provider.RACKSPACE_NOVA_LON: Provider.RACKSPACE,
-    Provider.RACKSPACE_NOVA_ORD: Provider.RACKSPACE
+    Provider.RACKSPACE_NOVA_ORD: Provider.RACKSPACE,
+
+    # AWS
+    Provider.EC2_US_EAST: Provider.EC2,
+    Provider.EC2_US_EAST_OHIO: Provider.EC2,
+    Provider.EC2_EU: Provider.EC2,
+    Provider.EC2_EU_WEST: Provider.EC2,
+    Provider.EC2_US_WEST: Provider.EC2,
+    Provider.EC2_AP_SOUTHEAST: Provider.EC2,
+    Provider.EC2_AP_SOUTHEAST2: Provider.EC2,
+    Provider.EC2_AP_NORTHEAST: Provider.EC2,
+    Provider.EC2_AP_NORTHEAST1: Provider.EC2,
+    Provider.EC2_AP_NORTHEAST2: Provider.EC2,
+    Provider.EC2_US_WEST_OREGON: Provider.EC2,
+    Provider.EC2_SA_EAST: Provider.EC2,
+    Provider.EC2_AP_SOUTHEAST: Provider.EC2,
+
+    # ElasticHosts
+    Provider.ELASTICHOSTS_UK1: Provider.ELASTICHOSTS,
+    Provider.ELASTICHOSTS_UK2: Provider.ELASTICHOSTS,
+    Provider.ELASTICHOSTS_US1: Provider.ELASTICHOSTS,
+    Provider.ELASTICHOSTS_US2: Provider.ELASTICHOSTS,
+    Provider.ELASTICHOSTS_US3: Provider.ELASTICHOSTS,
+    Provider.ELASTICHOSTS_CA1: Provider.ELASTICHOSTS,
+    Provider.ELASTICHOSTS_AU1: Provider.ELASTICHOSTS,
+    Provider.ELASTICHOSTS_CN1: Provider.ELASTICHOSTS,
 }
 
 
-class NodeState(object):
+class NodeState(Type):
     """
     Standard states for a node
 
     :cvar RUNNING: Node is running.
+    :cvar STARTING: Node is starting up.
     :cvar REBOOTING: Node is rebooting.
     :cvar TERMINATED: Node is terminated. This node can't be started later on.
+    :cvar STOPPING: Node is currently trying to stop.
     :cvar STOPPED: Node is stopped. This node can be started later on.
     :cvar PENDING: Node is pending.
     :cvar SUSPENDED: Node is suspended.
     :cvar ERROR: Node is an error state. Usually no operations can be performed
                  on the node once it ends up in the error state.
     :cvar PAUSED: Node is paused.
+    :cvar RECONFIGURING: Node is being reconfigured.
     :cvar UNKNOWN: Node state is unknown.
     """
-    RUNNING = 0
-    REBOOTING = 1
-    TERMINATED = 2
-    PENDING = 3
-    UNKNOWN = 4
-    STOPPED = 5
-    SUSPENDED = 6
-    ERROR = 7
-    PAUSED = 8
-
-    @classmethod
-    def tostring(cls, value):
-        values = cls.__dict__
-        values = dict([(key, string) for key, string in values.items() if
-                       not key.startswith('__')])
-
-        for item_key, item_value in values.items():
-            if value == item_value:
-                return item_key
-
-    @classmethod
-    def fromstring(cls, value):
-        return getattr(cls, value.upper(), None)
+    RUNNING = 'running'
+    STARTING = 'starting'
+    REBOOTING = 'rebooting'
+    TERMINATED = 'terminated'
+    PENDING = 'pending'
+    UNKNOWN = 'unknown'
+    STOPPING = 'stopping'
+    STOPPED = 'stopped'
+    SUSPENDED = 'suspended'
+    ERROR = 'error'
+    PAUSED = 'paused'
+    RECONFIGURING = 'reconfiguring'
+    MIGRATING = 'migrating'
+    NORMAL = 'normal'
 
 
-class StorageVolumeState(object):
+class StorageVolumeState(Type):
     """
     Standard states of a StorageVolume
     """
-    AVAILABLE = "available"
-    ERROR = "error"
-    INUSE = "in_use"
-    CREATING = "creating"
-    DELETING = "deleting"
-    DELETED = "deleted"
-    BACKUP = "backup"
-    ATTACHING = "attaching"
-    UNKNOWN = "unknown"
+    AVAILABLE = 'available'
+    ERROR = 'error'
+    INUSE = 'inuse'
+    CREATING = 'creating'
+    DELETING = 'deleting'
+    DELETED = 'deleted'
+    BACKUP = 'backup'
+    ATTACHING = 'attaching'
+    UNKNOWN = 'unknown'
+    MIGRATING = 'migrating'
 
 
-class VolumeSnapshotState(object):
+class VolumeSnapshotState(Type):
     """
     Standard states of VolumeSnapshots
     """
-    AVAILABLE = 0
-    ERROR = 1
-    CREATING = 2
-    DELETING = 3
-    RESTORING = 4
-    UNKNOWN = 5
+    AVAILABLE = 'available'
+    ERROR = 'error'
+    CREATING = 'creating'
+    DELETING = 'deleting'
+    RESTORING = 'restoring'
+    UNKNOWN = 'unknown'
 
 
 class Architecture(object):
